@@ -26,6 +26,7 @@
 #include <sys/stat.h>
 #include <unistd.h>
 
+#include "../libsnap-confine-private/cgroup-freezer-support.h"
 #include "../libsnap-confine-private/classic.h"
 #include "../libsnap-confine-private/cleanup-funcs.h"
 #include "../libsnap-confine-private/locking.h"
@@ -69,7 +70,7 @@ void sc_maybe_fixup_permissions()
 // https://forum.snapcraft.io/t/weird-udev-enumerate-error/2360/17
 void sc_maybe_fixup_udev()
 {
-	glob_t glob_res __attribute__ ((cleanup(globfree))) = {
+	glob_t glob_res SC_CLEANUP(globfree) = {
 	.gl_pathv = NULL,.gl_pathc = 0,.gl_offs = 0,};
 	const char *glob_pattern = "/run/udev/tags/snap_*/*nvidia*";
 	int err = glob(glob_pattern, 0, NULL, &glob_res);
@@ -95,7 +96,7 @@ int main(int argc, char **argv)
 {
 	// Use our super-defensive parser to figure out what we've been asked to do.
 	struct sc_error *err = NULL;
-	struct sc_args *args __attribute__ ((cleanup(sc_cleanup_args))) = NULL;
+	struct sc_args *args SC_CLEANUP(sc_cleanup_args) = NULL;
 	args = sc_nonfatal_parse_args(&argc, &argv, &err);
 	sc_die_on_error(err);
 
@@ -141,11 +142,10 @@ int main(int argc, char **argv)
 	}
 #endif
 
-	char *snap_context __attribute__ ((cleanup(sc_cleanup_string))) = NULL;
+	char *snap_context SC_CLEANUP(sc_cleanup_string) = NULL;
 	// Do no get snap context value if running a hook (we don't want to overwrite hook's SNAP_COOKIE)
 	if (!sc_is_hook_security_tag(security_tag)) {
-		struct sc_error *err
-		    __attribute__ ((cleanup(sc_cleanup_error))) = NULL;
+		struct sc_error *err SC_CLEANUP(sc_cleanup_error) = NULL;
 		snap_context = sc_cookie_get_from_snapd(snap_name, &err);
 		if (err != NULL) {
 			error("%s\n", sc_error_msg(err));
@@ -220,6 +220,12 @@ int main(int argc, char **argv)
 			// old version
 			sc_maybe_fixup_permissions();
 			sc_maybe_fixup_udev();
+
+			// Associate each snap process with a dedicated snap freezer
+			// control group. This simplifies testing if any processes
+			// belonging to a given snap are still alive.
+			// See the documentation of the function for details.
+			sc_cgroup_freezer_join(snap_name, getpid());
 			sc_unlock(snap_name, snap_lock_fd);
 
 			// Reset path as we cannot rely on the path from the host OS to
