@@ -289,20 +289,6 @@ ATTRS{idVendor}=="1c9e", ATTRS{idProduct}=="9605", ENV{ID_USB_INTERFACE_NUM}=="0
 LABEL="ofono_speedup_end"
 `
 
-/*
-  1.Linux modem drivers set up the modem device /dev/modem as a symbolic link
-    to the actual device to /dev/ttyS*
-  2./dev/socket/rild is just a socket, not device node created by rild daemon.
-    Similar case for chnlat*.
-  So we intetionally skipped modem, rild and chnlat.
-*/
-const ofonoPermanentSlotUDevTag = `
-KERNEL=="tty[A-Z]*[0-9]*|cdc-wdm[0-9]*", TAG+="###CONNECTED_SECURITY_TAGS###"
-KERNEL=="tun",          TAG+="###CONNECTED_SECURITY_TAGS###"
-KERNEL=="tun[0-9]*",    TAG+="###CONNECTED_SECURITY_TAGS###"
-KERNEL=="dsp",          TAG+="###CONNECTED_SECURITY_TAGS###"
-`
-
 type ofonoInterface struct{}
 
 func (iface *ofonoInterface) Name() string {
@@ -334,20 +320,24 @@ func (iface *ofonoInterface) AppArmorPermanentSlot(spec *apparmor.Specification,
 	return nil
 }
 
-func (iface *ofonoInterface) DBusPermanentSlot(spec *dbus.Specification, plug *interfaces.Plug, slot *interfaces.Slot) error {
+func (iface *ofonoInterface) DBusPermanentSlot(spec *dbus.Specification, slot *interfaces.Slot) error {
 	spec.AddSnippet(ofonoPermanentSlotDBus)
 	return nil
 }
 
 func (iface *ofonoInterface) UDevPermanentSlot(spec *udev.Specification, slot *interfaces.Slot) error {
-	old := "###CONNECTED_SECURITY_TAGS###"
-	udevRule := ofonoPermanentSlotUDev
-	for appName := range slot.Apps {
-		tag := udevSnapSecurityName(slot.Snap.Name(), appName)
-		udevRule += strings.Replace(ofonoPermanentSlotUDevTag, old, tag, -1)
-		spec.AddSnippet(udevRule)
-	}
-	spec.AddSnippet(udevRule)
+	spec.AddSnippet(ofonoPermanentSlotUDev)
+	/*
+	   1.Linux modem drivers set up the modem device /dev/modem as a symbolic link
+	     to the actual device to /dev/ttyS*
+	   2./dev/socket/rild is just a socket, not device node created by rild daemon.
+	     Similar case for chnlat*.
+	   So we intetionally skipped modem, rild and chnlat.
+	*/
+	spec.TagDevice(`KERNEL=="tty[A-Z]*[0-9]*|cdc-wdm[0-9]*"`)
+	spec.TagDevice(`KERNEL=="tun"`)
+	spec.TagDevice(`KERNEL=="tun[0-9]*"`)
+	spec.TagDevice(`KERNEL=="dsp"`)
 	return nil
 }
 
