@@ -98,17 +98,8 @@ func deriveContent(spec *Specification, snapInfo *snap.Info) map[string]*osutil.
 	}
 	fstate := &osutil.FileState{Content: buffer.Bytes(), Mode: 0644}
 	content := make(map[string]*osutil.FileState)
-	// Add the new per-snap fstab file. This file will be read by snap-confine.
+	// Add the per-snap fstab file. This file is read by snap-update-ns.
 	content[fmt.Sprintf("snap.%s.fstab", snapInfo.Name())] = fstate
-	// Add legacy per-app/per-hook fstab files. Those are identical but
-	// snap-confine doesn't yet load it from a per-snap location. This can be
-	// safely removed once snap-confine is updated.
-	for _, appInfo := range snapInfo.Apps {
-		content[fmt.Sprintf("%s.fstab", appInfo.SecurityTag())] = fstate
-	}
-	for _, hookInfo := range snapInfo.Hooks {
-		content[fmt.Sprintf("%s.fstab", hookInfo.SecurityTag())] = fstate
-	}
 	return content
 }
 
