@@ -25,7 +25,7 @@ import (
 	"github.com/snapcore/snapd/interfaces"
 	"github.com/snapcore/snapd/interfaces/ifacetest"
 	"github.com/snapcore/snapd/interfaces/udev"
-	"github.com/snapcore/snapd/snap"
+	"github.com/snapcore/snapd/snap/snaptest"
 )
 
 type specSuite struct {
@@ -55,22 +55,28 @@ var _ = Suite(&specSuite{
 			return nil
 		},
 	},
-	plug: &interfaces.Plug{
-		PlugInfo: &snap.PlugInfo{
-			Snap:      &snap.Info{SuggestedName: "snap1"},
-			Name:      "name",
-			Interface: "test",
-		},
-	},
-	slot: &interfaces.Slot{
-		SlotInfo: &snap.SlotInfo{
-			Snap:      &snap.Info{SuggestedName: "snap2"},
-			Name:      "name",
-			Interface: "test",
-		},
-	},
 })
 
+func (s *specSuite) SetUpSuite(c *C) {
+	info1 := snaptest.MockInfo(c, `name: snap1
+plugs:
+    name:
+        interface: test
+apps:
+    foo:
+        command: bin/foo
+hooks:
+    configure:
+`, nil)
+	info2 := snaptest.MockInfo(c, `name: snap2
+slots:
+    name:
+        interface: test
+`, nil)
+	s.plug = &interfaces.Plug{PlugInfo: info1.Plugs["name"]}
+	s.slot = &interfaces.Slot{SlotInfo: info2.Slots["name"]}
+}
+
 func (s *specSuite) SetUpTest(c *C) {
 	s.spec = &udev.Specification{}
 }
@@ -80,6 +86,24 @@ func (s *specSuite) TestAddSnippte(c *C) {
 	c.Assert(s.spec.Snippets(), DeepEquals, []string{"foo"})
 }
 
+func (s *specSuite) TestTagDevice(c *C) {
+	// TagDevice acts in the scope of the plug/slot (as appropriate) and
+	// affects all of the apps and hooks related to the given plug or slot
+	// (with the exception that slots cannot have hooks).
+	iface := &ifacetest.TestInterface{
+		InterfaceName: "test",
+		UDevConnectedPlugCallback: func(spec *udev.Specification, plug *interfaces.Plug, plugAttrs map[string]interface{}, slot *interfaces.Slot, slotAttrs map[string]interface{}) error {
+			spec.TagDevice(`kernel="voodoo"`)
+			return nil
+		},
+	}
+	c.Assert(s.spec.AddConnectedPlug(iface, s.plug, nil, s.slot, nil), IsNil)
+	c.Assert(s.spec.Snippets(), DeepEquals, []string{
+		`kernel="voodoo", TAG+="snap_snap1_foo"`,
+		`kernel="voodoo", TAG+="snap_snap1_hook_configure"`,
+	})
+}
+
 // The spec.Specification can be used through the interfaces.Specification interface
 func (s *specSuite) TestSpecificationIface(c *C) {
 	var r interfaces.Specification = s.spec
