@@ -19,11 +19,22 @@
 
 package ctlcmd
 
-import "fmt"
+import (
+	"fmt"
+	"github.com/snapcore/snapd/overlord/servicestate"
+	"github.com/snapcore/snapd/overlord/state"
+	"github.com/snapcore/snapd/snap"
+)
 
 var AttributesTask = attributesTask
 var CopyAttributes = copyAttributes
 
+func MockServicestateControlFunc(f func(*state.State, []*snap.AppInfo, *servicestate.Instruction) (*state.TaskSet, error)) (restore func()) {
+	old := servicestateControl
+	servicestateControl = f
+	return func() { servicestateControl = old }
+}
+
 func AddMockCommand(name string) *MockCommand {
 	mockCommand := NewMockCommand()
 	addCommand(name, "", "", func() command { return mockCommand })
